"""Valid FIDO 2 test data."""
from __future__ import unicode_literals

# Generated using examples/credential.py from python-fido2
HOSTNAME = 'testserver'
USERNAME = 'kryten'
USER_FIRST_NAME = 'Kryten'
USER_LAST_NAME = '2X4B-523P'
USER_FULL_NAME = USER_FIRST_NAME + ' ' + USER_LAST_NAME
REGISTRATION_CHALLENGE = "Rimmerstasks"
REGISTRATION_CLIENT_DATA = ('eyJ0eXBlIjogIndlYmF1dGhuLmNyZWF0ZSIsICJjbGllbnRFeHRlbnNpb25zIjoge30sICJjaGFsbGVuZ2UiOiAiUm'
                            'ltbWVyc3Rhc2tzIiwgIm9yaWdpbiI6ICJodHRwczovL3Rlc3RzZXJ2ZXIifQ==')
ATTESTATION_OBJECT = (
    'owFoZmlkby11MmYCWMTr2rckepjnyU3pMGbSTDiv2aDKKY42hLR6uxEx/MnyAEEAAAAAAAAAAAAAAAAAAAAAAAAAAABAVKZJP1j2MAl+NE9Ho0pa+T'
    'ybzzmoI4XbU/MvHCr7lQBJc3O5WvHF9B85RIk2ZEyRSwVHh8MW+te50I9juwr6gqUBAgMmIAEhWCDhTkbEO9WpfLwhmoLoPnka4fFANkMYCqhpxI6V'
    'jSDt1CJYIPirPpi/vXbFj/xN9eCGW+rTRQvplmahLa2SupgsLQkwA6Jjc2lnWEcwRQIgP4FTyNoVKsH4/f1TBlqJV5kRpIJR3Kof7N7hHygMyaECIQ'
    'DK+UaV3uOSS7QQ2p1OmDxwpOpakKwRGL1yVLq7QyLsE2N4NWOBWQJIMIICRDCCAS6gAwIBAgIEeMDfDjALBgkqhkiG9w0BAQswLjEsMCoGA1UEAxMj'
    'WXViaWNvIFUyRiBSb290IENBIFNlcmlhbCA0NTcyMDA2MzEwIBcNMTQwODAxMDAwMDAwWhgPMjA1MDA5MDQwMDAwMDBaMCoxKDAmBgNVBAMMH1l1Ym'
    'ljbyBVMkYgRUUgU2VyaWFsIDIwMjU5MDU5MzQwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAS1uHFcg/3+DqFcRXeshY30jBdv3oedyvS4PUDTIPJv'
    'reYl/Pf1yK/YNRj4254h7Ag7GEWAxxfsSkcLlopvuj9vozswOTAiBgkrBgEEAYLECgIEFTEuMy42LjEuNC4xLjQxNDgyLjEuMTATBgsrBgEEAYLlHA'
    'IBAQQEAwIFIDALBgkqhkiG9w0BAQsDggEBAD72q/ZKkWsL+ZSTjdyVNOBUQAJoVninLEOnq+ZdkGX/YfRRzoo67thmidGQuVCvAHpU0THu8G/ia06n'
    'uz4yt5IFpd+nYAQ0U+NK+ETDfNSoX4xcLYcOCiiyt+1EAkH9s3krIHaw4Yr6m0Mu7vwmWLoJBcQbJKk8bsi7ptVvM+jWU9fPa9UBVFWiZZdA99zFHM'
    'AxYJzQPqbN6Tmeygh2MpB2P7TI0A9WkGmhJUkAauuwaiGiFOSZmDe0KegdflbTOlSS3ToWHIKTlUCBqn7vdJw6Vj2919ujlcxHPkRpbUGRhcJDesg6'
    'wGTBy+RyJ/96G3fH1eoMNn1F9jC9mY1Zsm4=')
CREDENTIAL_ID = 'VKZJP1j2MAl+NE9Ho0pa+TybzzmoI4XbU/MvHCr7lQBJc3O5WvHF9B85RIk2ZEyRSwVHh8MW+te50I9juwr6gg=='
AUTHENTICATION_CHALLENGE = "Listerspants"
AUTHENTICATION_CLIENT_DATA = ('eyJ0eXBlIjogIndlYmF1dGhuLmdldCIsICJjbGllbnRFeHRlbnNpb25zIjoge30sICJjaGFsbGVuZ2UiOiAiTGlz'
                              'dGVyc3BhbnRzIiwgIm9yaWdpbiI6ICJodHRwczovL3Rlc3RzZXJ2ZXIifQ==')
AUTHENTICATOR_DATA = '69q3JHqY58lN6TBm0kw4r9mgyimONoS0ersRMfzJ8gABAAAAmA=='
SIGNATURE = 'MEUCIFM+4zYIqdomNBAuzWsijo0YJHm4gTJs5DPLCYnSnZoAAiEA4neUdBFkBZH7oFsc6F3GRjMrmqUWCN1I++SGOF7kV4E='
