# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-07 12:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_fido', '0003_add_authenticator'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='u2fdevice',
            name='user',
        ),
        migrations.DeleteModel(
            name='U2fDevice',
        ),
    ]
